@@ -1,5 +1,5 @@
-maxlike <- function(formula, rasters, points,
-                    starts, hessian=TRUE, na.action="na.omit", ...)
+maxlike <- function(formula, rasters, points, starts, hessian=TRUE,
+                    fixed, removeDuplicates=FALSE, na.action="na.omit", ...)
 {
     if(identical(formula, ~1))
         stop("At least one continuous covariate must be specified in the formula")
@@ -19,6 +19,11 @@ maxlike <- function(formula, rasters, points,
         cd.names <- layerNames(rasters)
         npix <- prod(dim(rasters)[1:2])
         cellID <- cellFromXY(rasters, points)
+        duplicates <- duplicated(cellID)
+        if(removeDuplicates) {
+            cellID <- unique(cellID)
+            npts <- length(cellID)
+            }
         x <- as.data.frame(matrix(extract(rasters, cellID), npts))
         z <- as.data.frame(matrix(getValues(rasters), npix))
         names(x) <- names(z) <- cd.names
@@ -50,7 +55,7 @@ maxlike <- function(formula, rasters, points,
     npars <- ncol(X)
     parnames <- colnames(X)
     if(!"(Intercept)" %in% parnames)
-        stop("The intercept must be estimated")
+        stop("The intercept must be estimated or fixed")
     if(missing(starts)) {
         starts <- rep(0, npars)
         names(starts) <- parnames
@@ -64,12 +69,35 @@ maxlike <- function(formula, rasters, points,
         -1*sum(log(psix/sum(psiz) + .Machine$double.eps))
         }
 
-    fm <- optim(starts, nll, hessian=hessian, ...)
+    is.fixed <- rep(FALSE, npars)
+    if(!missing(fixed)) {
+        if(length(fixed) != length(starts))
+            stop("fixed should be a vector with the same length as the number of parameters to be estimated")
+        if(sum(is.real(fixed)) < 1)
+            stop("fixed must contain at least one real value")
+        is.fixed <- !is.na(fixed)
+        if(sum(!is.fixed) < 1)
+            stop("you cannot fix all parameters in the model")
+        npars <- sum(!is.fixed)
+        nll.fix <- function(p) {
+            p[is.fixed] <- fixed[is.fixed]
+            do.call("nll", list(pars=p))
+        }
+        fm <- optim(starts, nll.fix, hessian=hessian, ...)
+        fm$par[is.fixed] <- fixed[is.fixed]
+    } else {
+        fm <- optim(starts, nll, hessian=hessian, ...)
+    }
+    not.fixed <- !is.fixed
+
     par <- fm$par
     if(hessian) {
-        vc <- try(solve(fm$hessian))
-        if(identical(class(vc), "matrix"))
-           se <- sqrt(diag(vc))
+        vcTry <- try(solve(fm$hessian[not.fixed, not.fixed]))
+        if(identical(class(vcTry), "matrix")) {
+            vc <- matrix(0, length(par), length(par))
+            vc[not.fixed, not.fixed] <- vcTry
+            se <- sqrt(diag(vc))
+        }
         else {
             vc <- matrix(NA, npars, npars)
             se <- rep(NA, npars)
@@ -78,10 +106,14 @@ maxlike <- function(formula, rasters, points,
         vc <- matrix(NA, npars, npars)
         se <- rep(NA, npars)
     }
+    dimnames(vc) <- list(parnames, parnames)
     aic <- 2*fm$value + 2*npars
+
+    fitted <- plogis(Z %*% par)
     out <- list(Est=cbind(Est=par, SE=se), vcov=vc, AIC=aic, call=call,
                 pts.removed=pts.removed, pix.removed=pix.removed,
-                optim=fm)
+#                duplicates=duplicates,
+                optim=fm, not.fixed=not.fixed)
     class(out) <- c("maxlikeFit", "list")
     return(out)
     }
